#!/usr/bin/perl -w 
#  System        : combine-tk-pdfs.pl : 
#  Created By    : Dr. Detlef Groth, MPIMG Berlin, Germany
#  Created       : Mon Nov 27 20:47:43 2006
#  Last Modified : <070530.0638>
#  $Log$
##############################################################################

use Tk ;
use File::Basename;
use FindBin;
use lib "$FindBin::Bin/../lib";
require Tk::Tree;
require Tk::FileSelect ;
require Tk::Entry;
require Tk::Balloon;

require CombinePDFs ;
our %gui ; # keeps all gui widgets
our %v   ; # keeps all textvariables for the entry widgets
our @files ;
our $x = 0 ; # keps the number of files adedd
our $f = 0 ; # keeps the number of folders added

our $current_path ;
our $copy_path ;
our $last_dir = "." ;# keeps track of the last selected pdf file
$last_dir = $ENV{'HOME'}; 
our $filetypes = [['PDF Files',['*.pdf']],['All Files', ['*.*']]];
our $session_filetypes = [['Session Files',['*.sess']],['All Files', ['*.*']]];
our $bookmark_filetypes = [['Bookmark Files',['*.cnt']],['All Files', ['*.*']]];
our $openfolderimage ;
our $folderimage ;
our $VERSION = "1.2" ;
our $AUTHOR = "Dr. Detlef Groth" ;
our $YEAR = "2007" ;
our $APP = "dgPDFCombiner - 2007-06-02" ;

sub Row_Add {
    
    my $sel_path = $gui{tree}->infoSelection(); 
    if (!$sel_path) {
        $gui{mw}->messageBox(-message => "You must select an item!", -icon => 'info', -title  => 'Info!');
        return ;
    }
    my $file = $gui{mw}->getOpenFile(-initialdir => $last_dir, -filetypes => $filetypes);
    my $parent_path = @{$sel_path}[0];
    $parent_path =~ s/\..+// ;
    if ($file) {
        my $fname = $file ;
        $fname =~ s|.+[/\\]||g ;
        $v{filename} = $file ;
        $v{pages} = "all" ;
        $v{title} = $fname ;
        $x++ ;
        $last_dir = dirname($file) ;
        $path = "$parent_path.x$x" ;
        $gui{tree}->add($path);
        $gui{tree}->itemCreate($path,0,-itemtype => 'image', -image => $gui{mw}->Getimage("file"));
        $gui{tree}->itemCreate($path,1,-text => $v{filename});
        $gui{tree}->itemCreate($path,2,-text => $v{pages});
        $gui{tree}->itemCreate($path,3,-text => $v{title});
        $gui{tree}->selectionClear();
        $gui{tree}->selectionSet($path);
        on_open_folder($gui{tree}->infoParent($path));
        $gui{tree}->autosetmode;
        $current_path = $path ;
    }
}
sub Entry_Update {
    my ($path) = @_ ;
    $gui{tree}->selectionClear();
    $gui{tree}->selectionSet($path);
    $gui{tree}->focus ;
    $v{filename} = $gui{tree}->itemCget($path,1,-text);
    $v{pages} = $gui{tree}->itemCget($path,2,-text);
    $v{title} = $gui{tree}->itemCget($path,3,-text);
    $current_path = $path ;
    
}
sub Row_Select {
    my $path = $gui{tree}->infoSelection() ;
    if ($path) {
        $v{filename} = $gui{tree}->itemCget($path,1,-text);
        $v{pages} = $gui{tree}->itemCget($path,2,-text);
        $v{title} = $gui{tree}->itemCget($path,3,-text);
        $current_path = $path ;
    }
}
sub Row_Update {
    $gui{tree}->itemConfigure($current_path,1,-text => $v{filename});
    $gui{tree}->itemConfigure($current_path,2,-text => $v{pages});
    $gui{tree}->itemConfigure($current_path,3,-text => $v{title});
    return 1 ;
}
sub Row_Delete {
    $current_path = $gui{tree}->infoSelection();
    if ($current_path) {
        if($gui{mw}->messageBox(-message => "Really Delete Item\n$v{title}?", -icon => 'question', 
                                -title  => 'Please Answer!', -type => "YesNo")  eq "Yes") {
            $gui{tree}->delete('entry',$current_path);
            $current_path = "" ;
            $gui{tree}->selectionClear();
        }
    }
}
sub Row_Copy {
    $copy_path = $gui{tree}->infoSelection(); 
}
sub Row_Paste {
    if ($copy_path ne "") {
        $current_path = $gui{tree}->infoSelection(); 
        my $parent = $gui{tree}->infoParent($current_path);
        my $copy_parent = $gui{tree}->infoParent($copy_path);
        if ($copy_parent) {
         
            my $path ;
            if ($parent) {
                # we copy a file entry after the current file entry
                $x++ ;
                $path = "$parent.x$x";
                $gui{tree}->add($path, -after => $current_path); 
                on_open_folder($gui{tree}->infoParent($path));
            } else {
                # we copy a file item into the current selected folder
                $x++;
                $path = "@{$current_path}[0].x$x" ;
                $gui{tree}->add($path); 
                on_open_folder($gui{tree}->infoParent($path));
            }
            $gui{tree}->itemCreate($path,0,-itemtype => "image", -image => $gui{mw}->Getimage("file"));
            $gui{tree}->itemCreate($path,1,-text => $gui{tree}->itemCget($copy_path,1,-text));
            $gui{tree}->itemCreate($path,2,-text => $gui{tree}->itemCget($copy_path,2,-text));
            $gui{tree}->itemCreate($path,3,-text => $gui{tree}->itemCget($copy_path,3,-text));
            $current_path = $path ;
            $gui{tree}->selectionClear();
            $gui{tree}->selectionSet($path);
            $gui{tree}->autosetmode;
        } 
    } 
        
}
sub Row_Move {
    my $mode = shift ;
    $current_path = $gui{tree}->infoSelection();
    if ($current_path) {
        my $prev ;
        if ($mode eq "up") {
            $prev = $gui{tree}->infoPrev($current_path);
        } else {
            $prev = $gui{tree}->infoNext($current_path);
        }
        my $parent = $gui{tree}->infoParent($current_path);
        if ($parent && $prev) {
            # a file
            $x++ ;
            my $parent_path = $gui{tree}->infoParent($current_path);
            
            my $path = "$parent_path.x$x";
            if ($mode eq "up") {
                $gui{tree}->add($path, -before => $prev); 
            } else {
                $gui{tree}->add($path, -after => $prev);  
            }
            $gui{tree}->itemCreate($path,0,-itemtype => "image", -image => $gui{mw}->Getimage("file"));
            $gui{tree}->itemCreate($path,1,-text => $gui{tree}->itemCget($current_path,1,-text));
            $gui{tree}->itemCreate($path,2,-text => $gui{tree}->itemCget($current_path,2,-text));
            $gui{tree}->itemCreate($path,3,-text => $gui{tree}->itemCget($current_path,3,-text));
            $gui{tree}->delete("entry",$current_path);
            $current_path = $path ;
            $gui{tree}->selectionSet($path);
        } elsif ($prev) {
            #  a folder
            $f++ ;
            my $path = "f$f" ;
            if ($prev =~ /\./) {
                if ($mode eq "up") {
                    $prev = $gui{tree}->infoParent($prev);
                } else {
                    while ($prev ne "" and $prev =~ m/\./) {
                        $prev = $gui{tree}->infoNext($prev);
                    }
                }
            }
            return unless ($prev);
            if ($mode eq "up") {
                $gui{tree}->add($path, -before => $prev); 
            } else {
                $gui{tree}->add($path, -after => $prev); 
            }
            $gui{tree}->itemCreate($path,0,-itemtype => "image", -image => $openfolderimage);
            $gui{tree}->itemCreate($path,1,-text => $gui{tree}->itemCget($current_path,1,-text));
            $gui{tree}->itemCreate($path,2,-text => $gui{tree}->itemCget($current_path,2,-text));
            $gui{tree}->itemCreate($path,3,-text => $gui{tree}->itemCget($current_path,3,-text));
          
            my @childs = $gui{tree}->infoChildren($current_path);
            foreach my $child (@childs) {
                $x++ ;
                my $cpath = "$path.x$x";
                $gui{tree}->add($cpath); 
                $gui{tree}->itemCreate($cpath,0,-itemtype => "image", -image => $gui{mw}->Getimage("file"));
                $gui{tree}->itemCreate($cpath,1,-text => $gui{tree}->itemCget($child,1,-text));
                $gui{tree}->itemCreate($cpath,2,-text => $gui{tree}->itemCget($child,2,-text));
                $gui{tree}->itemCreate($cpath,3,-text => $gui{tree}->itemCget($child,3,-text));
                $gui{tree}->delete("entry",$child);
            }
            $gui{tree}->delete("entry",$current_path);
            $current_path = $path ;
            $gui{tree}->selectionSet($path);
            
        } 
        $gui{tree}->autosetmode();
    }
}
sub Row_MoveUp {
    Row_Move("up");
    return ;
}
sub Row_MoveDown {
    Row_Move("down");
    return ;
}
sub Tk::Error
{
    my ($widget,$error,@locations) = @_;
    $gui{mw}->messageBox(-message => "$error\nPlease submit a bug report to dgroth\@gmx.de!", -icon => 'error', -title  => 'Error!');
    #print "my error: $widget,$error,@locations\n" ;

}
sub GUI_Build {
    my $r = 0 ;
    $gui{mw} = MainWindow->new;
    print "\nThis window is for debugging purposes, please don't close it!\n";
     $gui{mw}->iconimage($gui{mw}->Photo(-file => "$FindBin::RealBin/favicon32.gif"));
    $gui{mw}->title("$APP");
   
    $gui{top} = $gui{mw}->toplevel;
    $gui{mnu} = $gui{top}->Menu(-type => 'menubar');
    $gui{top}->configure(-menu => $gui{mnu});
    my $f = $gui{mnu}->cascade(-label => '~File', -tearoff => 0);
    $f->command(-label => 'Add Folder ...', -underline => 0,  -command => [\&Folder_Add, 'Add Folder']);
    $f->command(-label => 'Add PDF-File ...', -underline => 1,  -command => [\&Row_Add, 'Add PDF-File']);
    $f->command(-label => 'Create PDF-File ...', -underline => 0,  -command => [\&PDF_Create, 'Create']);
    $f->separator;
    $f->command(-label => 'New Session', -underline => 0,  -command => [\&Session_New, 'New Session']);
    $f->command(-label => 'Restore Session ...', -underline => 0,  -command => [\&Session_Restore, 'Session Restore']);
    $f->command(-label => 'Save Session ...', -underline => 0,  -command => [\&Session_Save, 'Session Save']);
    $f->separator;
    $f->command(-label => 'Load Bookmarks File ...', -underline => 0,  -command => [\&Bookmarks_Load, 'Bookmarks Load']);
    $f->separator;

    $f->command(-label => 'Exit' , -underline => 1, -command => sub { exit 0 });
    my $h = $gui{mnu}->cascade(-label => '~Help', -tearoff => 0);
    $h->command(-label => 'About', -underline => 0,  -command => [\&Help_About, 'About']);
    $h->command(-label => 'Keys', -underline => 0,  -command => [\&Help_Keys, 'Keys']);
    $h->command(-label => 'License', -underline => 0,  -command => [\&Help_License, 'License']);
    $h->command(-label => 'License2', -underline => 0,  -command => \&Help_License2);
   
    $gui{tree}=$gui{mw}->Scrolled("Tree", -columns => 4, -header => 1,
                                  -opencmd => \&on_open_folder ,
                                  -closecmd => \&on_close_folder,
                                  )
    ->pack(-side => 'top', -fill => 'both', -expand => "yes"); # , -command => \&Entry_Update
    $gui{tree}->headerCreate(0,-text => " ");
    $gui{tree}->headerCreate(1,-text => "File");
    $gui{tree}->headerCreate(2,-text => "Pages");
    $gui{tree}->headerCreate(3,-text => "Title");
    $openfolderimage = $gui{tree}->Getimage("openfolder") ;
    $folderimage = $gui{tree}->Getimage("folder") ;
    Folder_Add();
    $gui{frame} = $gui{mw}->Frame()->pack(-side=> "top", -fill => "y", -expand => 0, -ipadx => 30);
    $gui{ef} = $gui{frame}->Entry(-textvariable => \$v{filename}, -width => 50)->grid(-row => $r, -column => 0);
    $gui{ep} = $gui{frame}->Entry(-textvariable => \$v{pages}
                                  )->grid(-row => $x, -column => 1);
    $gui{et} = $gui{frame}->Entry(-textvariable => \$v{title}, -width => 40)->grid(-row => $r, -column => 2);
    $gui{frame}->Button(-text => "Update",-command => \&Row_Update)->grid(-row => $r, -column => 3);
#    $gui{frame_sponsor} = $gui{mw}->Frame()->pack(-side=> "top", -fill => "x", -expand => 0, -ipadx => 10, -pady => 10);
    #$gui{frame_sponsor_left} = $gui{frame_sponsor}->Frame()->grid(-row => 0, -column => 0); #side=> "top", -fill => "x", -expand => 0, -ipadx => 30);
#    $gui{frame_sponsor_left_top} = $gui{frame_sponsor}->Button( -activebackground => "#ffff00", -background => "#000000", -command => \&Browser_Run, -text => "                 ", -width => 100)->pack(-side=> "top", -fill => "x", -expand => 1, -padx => 0);
#    $gui{frame_sponsor_left_bottom} = $gui{frame_sponsor}->Button(-activebackground => "#000000", -background =>  "#ffff00", -command => \&Browser_Run, -text => "                 ")->pack(-side=> "bottom", -fill => "both", -expand => 1);
    $gui{balloon} = $gui{mw}->Balloon();
    $gui{balloon}->attach($gui{ep},-balloonmsg => "Page ranges like `1-3,4,5-7` or `all`!");
    $gui{balloon}->attach($gui{et},-balloonmsg => "Title that will appear in the bookmarks panel!");
    $gui{balloon}->attach($gui{ef},-balloonmsg => "Valid pdf filepath!");
#    foreach my $lr (qw(left)) {
#        foreach my $tb (qw(top bottom)) {
#            my $key = "frame_sponsor_${lr}_${tb}" ;
#            $gui{balloon}->attach($gui{$key},-balloonmsg => "Sponsor\n==============\nhttp://www.drweb.de");
#        }
#    }
    $gui{mw}->bind("<Control-u>", \&Row_MoveUp);
    $gui{mw}->bind("<Control-d>", \&Row_MoveDown);
    $gui{mw}->bind("<Control-c>", \&Row_Copy);
    $gui{mw}->bind("<Control-v>", \&Row_Paste);
    #$gui{mw}->bind("<Control-e>", \&Perl_Eval);
    $gui{tree}->bind("<Delete>", \&Row_Delete);
    $gui{tree}->bind("<Button-1>",\&Row_Select);
    
    MainLoop;
}
sub Browser_Run {
    my $url = "http://www.drweb.de/webmaster/pdf-kombinieren-software.shtml";
    if ((eval "system(\"start $url\")") == -1) {
        if ((eval "system(\"rundll32.exe url.dll,FileProtocolHandler $url\")") == -1) {
            if ((eval "system(\"firefox $url\")") == -1 || system(\"mozilla $url\")") == -1) {
                $gui{mw}->messageBox(-message => "$APP $YEAR\nPlease visit pour Sponsor at \nhttp://www.drweb.de", -icon => 'info', -title  => 'About!');
            }
        }
    }
}
sub Help_License {
    my $message = "Copyright 2007, Dr. Detlef Groth, dgroth(at)gmx.de\nThis work is licensed under a \nCreative Commons Attribution-NoDerivs 2.0 Germany License.\nhttp://creativecommons.org/licenses/by-nd/2.0/de/\n";
    $message .= "This software comes with no warranty of any kind!\n";
    $message .="NO WARRANTIES. NO LIABILITY FOR DAMAGES.";
    $gui{mw}->messageBox(-message => "$message", -icon => 'info', -title  => 'License'); 
    
}
sub Help_About {
   $gui{mw}->messageBox(-message => "GUI - $APP $YEAR\n\@ Dr. $AUTHOR\nhttp://www.dgroth.de/dgpdfcombiner/", -icon => 'info', -title  => 'About!'); 
}
sub Help_Keys {
    $gui{mw}->messageBox(-message => "with selected item:\nC-u = move up\nC-d = move down\nDel = delete\nC-c = copy file item\nC-v = paste file item\n", -icon => 'info', -title  => 'Keys!');
}
sub Folder_Add {
    $f++ ;
    my $path = "f$f";
    $gui{tree}->add("$path");
    $gui{tree}->itemCreate("$path",0,-itemtype => "image", 
                           -image => $gui{mw}->Getimage("folder"));
    $gui{tree}->itemCreate("$path",1, -text => "Folder");
    $gui{tree}->itemCreate("$path",2, -text => "none");
    $gui{tree}->itemCreate("$path",3, -text => "Document");
    $gui{tree}->selectionSet($path);
}
sub Bookmarks_Load {
    my $infile = $gui{mw}->getOpenFile(-defaultextension => "*.cnt",
                                       -filetypes => $bookmark_filetypes,
                                       -initialdir => $last_dir);
    return if (!$infile);
    $last_dir = dirname($infile) ;
    my $pdffile = $infile ;
    $pdffile =~ s/.cnt$/.pdf/i;
    if (!-r $pdffile) {
        $gui{mw}->messageBox(-message => "Pdf file $pdffile does not exists!", -icon => 'error', -title  => 'Error""'); 
        return ;
    }
    $gui{tree}->delete("all");
    open(INFILE, "$infile")  or $gui{mw}->messageBox(-message => "Can't open $infile!", -icon => 'error', -title  => 'Error""') and return ;  
    $f = 0 ;
    $x = 0 ;
    my $ppath ;
    my $path ;
    my $lastpage = 1 ;
    my @bookmarks = ();
    while (my $line = <INFILE>) {
        if ($line =~ /^([0-1]) +"(.+)" +([0-9]+)/) {
            my @arr = ($1,$2,$3);
            push(@bookmarks,\@arr);
        }
    }
    my $next = 0 ;
    foreach my $bm (@bookmarks) {
        $next++ ;
        my ($level,$title,$page) ;
        $level = $$bm[0];
        $title = $$bm[1];
        $page = $$bm[2];
        my $endpage = 1;
        if ($next == scalar(@bookmarks)) {
            $endpage = "end" ;
        } else {
            $endpage = $bookmarks[$next]->[2]-1;
        }
        if ($endpage eq "0") { $endpage = 1 }
        if ($level == 0) {
            $f++ ;
            $ppath = "f$f";
            $gui{tree}->add("$ppath");
            $gui{tree}->itemCreate("$ppath",0,
                                   -itemtype => "image",
                                   -image => $openfolderimage);
            $gui{tree}->itemCreate("$ppath",1, -text => "Folder");
            $gui{tree}->itemCreate("$ppath",2, -text => "$page-$endpage");
            $gui{tree}->itemCreate("$ppath",3, -text => $title);
        } elsif ($level == 1) {
            $x++ ;
            $path = "$ppath.x$x" ;
            $gui{tree}->add($path);
            $gui{tree}->itemCreate($path,0,
                                   -itemtype => "image", 
                                   -image => $gui{mw}->Getimage("file"));
            $gui{tree}->itemCreate($path,1, -text => $pdffile);
            $gui{tree}->itemCreate($path,2, -text => "$page-$endpage");
            $gui{tree}->itemCreate($path,3, -text => $title);
        }
        $lastpage = $page+1 ;
    }
    close INFILE ;
    $gui{tree}->autosetmode;
}  
sub Bookmarks_Save {
    my $pdffile = shift ;
    my $refBookmarks = shift ;
    my $refPagesParts = shift ;
    my $cntfile = $pdffile ;
    $cntfile =~ s/\.pdf/.cnt/i ;
    open(OUTFILE, ">$cntfile") ;
    
    my $x = 0 ;
    my $p = 0 ;
    foreach my $bm (@{$refBookmarks}) {
        $bm =~ s/\t/ "/;
        $bm =~ s/[\n\r]//g ;
        $p = $refPagesParts->[$x];
        $p++ ;
        print OUTFILE "$bm\" $p\n";
        $x++;
#        if ($bm =~ /^1/) {
#            $x++;
#        }
       
    }
    close OUTFILE ;
}
sub Session_New {
    my @folders = $gui{tree}->infoChildren();
    if (scalar(@folders) > 0) {
        my $answer = $gui{mw}->messageBox(
                                          -message => "Really delete old entries and start new session?", 
                                          -icon => 'question', -title  => 'Are your sure ?', 
                                          -type => "YesNo");
        if ($answer) {
            $gui{tree}->delete("all");
        }
    }
    
}
sub Session_Restore {
    my $infile = $gui{mw}->getOpenFile(-defaultextension => "*.sess",
                                       -filetypes => $session_filetypes,
                                       -initialdir => $last_dir);
    return if (!$infile);
    $gui{tree}->delete("all");
    open(INFILE, "$infile") ;
    $f = 0 ;
    $x = 0 ;
    my $ppath ;
    my $path ;
    while (my $line = <INFILE>) {
        $line =~ s/\s+$//;
        my @F = split(/\t/,$line);
        if ($F[0] == 0) {
            $f++ ;
            $ppath = "f$f";
            $gui{tree}->add("$ppath");
            $gui{tree}->itemCreate("$ppath",0,
                                   -itemtype => "image",
                                   -image => $openfolderimage);
            $gui{tree}->itemCreate("$ppath",1, -text => $F[1]);
            $gui{tree}->itemCreate("$ppath",2, -text => $F[2]);
            $gui{tree}->itemCreate("$ppath",3, -text => $F[3]);
        } elsif ($F[0] == 1) {
            $x++ ;
            $path = "$ppath.x$x" ;
            $gui{tree}->add($path);
            $gui{tree}->itemCreate($path,0,
                                   -itemtype => "image", 
                                   -image => $gui{mw}->Getimage("file"));
            $gui{tree}->itemCreate($path,1, -text => $F[1]);
            $gui{tree}->itemCreate($path,2, -text => $F[2]);
            $gui{tree}->itemCreate($path,3, -text => $F[3]);
            
        }
    }
    close INFILE ;
    $gui{tree}->autosetmode;
}
sub Session_Save {
   
    my $outfile = $gui{mw}->getSaveFile(-defaultextension => "*.sess",
                                        -filetypes => $session_filetypes,
                                        -initialdir => $last_dir);
    return if (!$outfile);
    open(OUTFILE, ">$outfile") ;
    my @folders = $gui{tree}->infoChildren();
    foreach my $folder (@folders) {
        print OUTFILE "0";
        foreach my $x (1..3) {
            print OUTFILE "\t". $gui{tree}->itemCget($folder,$x,-text);
        }
        print OUTFILE "\n" ;
        my @childs = $gui{tree}->infoChildren($folder);
        foreach my $child (@childs) {
            print OUTFILE "1";
            foreach my $y (1..3) {
                print OUTFILE "\t". $gui{tree}->itemCget($child,$y,-text);
            }
            print OUTFILE "\n" ;
        }
    }
    close OUTFILE ;
}
sub Check_File {
    my $file = shift ;
    if ($file eq "Folder") {
        return 1 ;
    }
    if (!-e $file) {
        $gui{mw}->messageBox(-message => "File `$file` does not exists!", 
                             -icon => 'error', -title  => 'Error!');
        return 0 ;
    }
    return 1 ;
}
sub Check_Pages {
    my $p = shift ;
    my $file = shift ;
    foreach my $pages (split(/,/,$p)) {
        if ($pages !~ m/^[0-9]+-([0-9]+|end)$/ and $pages !~ m/^[0-9]+$/ and $pages ne "all") {
            $gui{mw}->messageBox(-message => "Invalid page definition for $file $p '$pages'!\n     Must be either 'from-to' or 'all'!\n", 
                                 -icon => 'error',
                                 -title => "Error!");
            return 0;
        }
    }
    return 1 ;
}
sub PDF_Create {
    my @pages ;
    my @infiles ;
    my @bookmarks;
    my $outfile;
    my @folders = $gui{tree}->infoChildren();
    my $x = 0 ;
    my $file ;
    foreach my $folder (@folders) {
        push(@bookmarks, "0\t" . $gui{tree}->itemCget($folder,3,-text));
        $file = $gui{tree}->itemCget($folder,1,-text);
        return unless Check_File($file);
        if (-e $file) {
            push (@infiles, $file);
            my $p = $gui{tree}->itemCget($folder,2,-text);
            return unless Check_Pages($p,$file);
            push (@pages,$p);
        } else {
            push (@infiles,"Folder");
            push (@pages,"None");
        }
        my @childs = $gui{tree}->infoChildren($folder);
        foreach my $child (@childs) {
            $x++ ;
            $file = $gui{tree}->itemCget($child,1,-text);
            return unless Check_File($file);
            if (-e $file) {
                push (@infiles, $file);
                my $p = $gui{tree}->itemCget($child,2,-text);
                return unless Check_Pages($p,$file);
                push (@pages,$p);
                my $title = $gui{tree}->itemCget($child,3,-text);
                push (@bookmarks,"1\t$title");
            }
        }
    }
    if ($x > 0) {
        $outfile = $gui{mw}->getSaveFile(-defaultextension => "*.pdf",
                                            -filetypes => $filetypes,
                                            -initialdir => $last_dir);
        if ($outfile) {
            CombinePDFs::createPDF(\@infiles,\@pages,"${outfile}.tmp"); 
            CombinePDFs::addBookmarksByArray("${outfile}.tmp",$outfile,@bookmarks);
            my @parts =  CombinePDFs::getPagesParts();
            Bookmarks_Save($outfile,\@bookmarks,\@parts);
            if (-e $outfile) {
                $gui{mw}->messageBox(-message => "Creating file `$outfile` done!", 
                                     -icon => 'info',
                                     -title => "Ok!");
            } else {
                $gui{mw}->messageBox(-message => "Error creating file `$outfile`!", 
                                     -icon => 'error',
                                     -title => "Error!");
            }
            unlink "$outfile.tmp" if (-e "$outfile.tmp") ;
        }
    }
}
sub on_open_folder {
    my $item = $_[0] ;
    $gui{tree}->itemConfigure(  $item ,0,  -image => $openfolderimage); 
    foreach my $child ($gui{tree}->infoChildren($item)) {
        $gui{tree}->show(-entry => $child);
    }
}
sub on_close_folder {
    my $item = $_[0] ;
    $gui{tree}->itemConfigure( $item ,0,  -image => $folderimage); 
    foreach my $child ($gui{tree}->infoChildren($item)) {
        $gui{tree}->hide(-entry => $child); 
    }
}
GUI_Build;
